﻿using System;
using System.Text;
using System.Runtime.Serialization.Json;
using System.IO;
using System.Net;
using System.Threading;
using System.Reflection;
using System.Web;
using System.Collections.Generic;
using GoogleTranslate;
using ICallisto;

namespace CallistoX
{
    static class Translator
    {
        private static Thread thread;
        private static bool busy = false;

        public static void Translate(IUser userobj, String text, TranslatorLanguages from, TranslatorLanguages to)
        {
            if (busy) // let's have just 1 translation at a time
                return;

            thread = new Thread(new ParameterizedThreadStart(Translate));

            TranslatorCredentials url = new TranslatorCredentials
            {
                Vroom = userobj.Vroom,
                Name = userobj.Name,
                Text = text,
                From = from.ToGoogleCodeString(),
                To = to.ToGoogleCodeString()
            };

            thread.Start(url);
        }

        private static void Translate(object args)
        {
            busy = true;

            try
            {
                TranslatorCredentials creds = args as TranslatorCredentials;
                WebRequest request = WebRequest.Create(creds.ToString());
                WebResponse response = request.GetResponse();
                Stream stream = response.GetResponseStream();
                StreamReader reader = new StreamReader(stream, Encoding.UTF8);
                String json = reader.ReadToEnd();
                reader.Close();
                stream.Flush();
                stream.Close();
                response.Close();
                Translation result = Deserialize<Translation>(json);
                IUser user = Callisto.FindUser(creds.Name);

                if (user != null)
                {
                    String str = HttpUtility.HtmlDecode(result.responseData.translatedText);

                    if (str.Length > 0)
                        user.SendText(str);
                }
            }
            catch { }

            busy = false;
        }

        private static Translation Deserialize<T>(string json)
        {
            Translation obj = Activator.CreateInstance<Translation>();
            MemoryStream ms = new MemoryStream(Encoding.UTF8.GetBytes(json));
            DataContractJsonSerializer serializer = new DataContractJsonSerializer(obj.GetType());
            obj = (Translation)serializer.ReadObject(ms);
            ms.Close();
            return obj;
        }

        private static string ToGoogleCodeString(this TranslatorLanguages value)
        {
            Type type = value.GetType();
            FieldInfo fieldInfo = type.GetField(value.ToString());
            GoogleTranslateCodeAttribute[] attribs = fieldInfo.GetCustomAttributes(
                typeof(GoogleTranslateCodeAttribute), false) as GoogleTranslateCodeAttribute[];
            return attribs.Length > 0 ? attribs[0].Value : null;
        }
    }

    class TranslatorCredentials
    {
        public int Vroom { get; set; }
        public String Name { get; set; }
        public String Text { get; set; }
        public String From { get; set; }
        public String To { get; set; }

        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("http://ajax.googleapis.com/ajax/services/language/translate?v=1.0&q=");
            sb.Append(Uri.EscapeDataString(this.Text));
            sb.Append("&langpair=");
            sb.Append(this.From + "|" + this.To);
            return sb.ToString();
        }
    }

    enum TranslatorLanguages : int
    {
        [GoogleTranslateCode("ar")]
        Arabic = 0,
        [GoogleTranslateCode("da")]
        Danish = 1,
        [GoogleTranslateCode("nl")]
        Dutch = 2,
        [GoogleTranslateCode("en")]
        English = 3,
        [GoogleTranslateCode("fr")]
        French = 4,
        [GoogleTranslateCode("de")]
        German = 5,
        [GoogleTranslateCode("it")]
        Italian = 6,
        [GoogleTranslateCode("pt")]
        Portuguese = 7,
        [GoogleTranslateCode("es")]
        Spanish = 8        
    }

    class GoogleTranslateCodeAttribute : Attribute
    {
        private String value;

        public GoogleTranslateCodeAttribute(String value)
        {
            this.value = value;
        }

        public String Value
        {
            get { return this.value; }
        }
    }

    class TranslatorManager
    {
        private static List<TranslatorUserItem> items = new List<TranslatorUserItem>();

        public static bool IsTranslated(IUser userobj)
        {
            return items.Find(x => x.Name == userobj.Name && x.Guid == userobj.Guid) != null;
        }

        public static bool Translate(IUser userobj, String text)
        {
            TranslatorUserItem i = items.Find(x => x.Name == userobj.Name && x.Guid == userobj.Guid);

            if (i != null)
            {
                Translator.Translate(userobj, text, i.From, i.To);
                return true;
            }

            return false;
        }

        public static void AddItem(IUser admin, IUser target, String args)
        {
            int t, f;

            if (args.Length != 3)
                return;

            if (!int.TryParse(args.Substring(0, 1), out f))
                return;

            if (!int.TryParse(args.Substring(2, 1), out t))
                return;

            if (f < 0 || f > 8)
                return;

            if (t < 0 || t > 8)
                return;

            TranslatorUserItem i = new TranslatorUserItem
            {
                Name = target.Name,
                Guid = target.Guid,
                From = (TranslatorLanguages)f,
                To = (TranslatorLanguages)t
            };

            TranslatorUserItem exists = items.Find(x => x.Name == target.Name && x.Guid == target.Guid);

            if (exists != null)
            {
                exists.From = i.From;
                exists.To = i.To;
            }
            else items.Add(i);

            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;

            if (StringTemplate.GetString(STType.AdminCommands, 22) != null)
                Callisto.Print(StringTemplate.GetString(STType.AdminCommands, 22).Replace("+t", target.Name).Replace("+n", _admin).Replace("+x", i.To.ToString()), true);

            AdminLog.AddLine(target.Name + " translated by " + admin.Name);
        }

        public static void RemoveItem(IUser admin, IUser target)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;

            if (items.RemoveAll(x => x.Name == target.Name && x.Guid == target.Guid) > 0)
            {
                if (StringTemplate.GetString(STType.AdminCommands, 23) != null)
                    Callisto.Print(StringTemplate.GetString(STType.AdminCommands, 23).Replace("+t", target.Name).Replace("+n", _admin), true);

                AdminLog.AddLine(target.Name + " untranslated by " + admin.Name);
            }
        }

        public static void ListLang(IUser admin)
        {
            for (int i = 0; i < 9; i++)
                Callisto.Print(admin, i + " - " + ((TranslatorLanguages)i).ToString());
        }
    }

    class TranslatorUserItem
    {
        public String Name { get; set; }
        public String Guid { get; set; }
        public TranslatorLanguages To { get; set; }
        public TranslatorLanguages From { get; set; }
    }
}

namespace GoogleTranslate
{
    [Serializable]
    public class JSONResponse
    {
        public string responseDetails;
        public string responseStatus;
    }

    [Serializable]
    public class Translation : JSONResponse
    {
        public TranslationResponseData responseData = new TranslationResponseData();
    }

    [Serializable]
    public class TranslationResponseData
    {
        public string translatedText;
    }
}
